""" Python Obit FITS file directory utilities
"""
# $Id: FITSDir.py,v 1.2 2007/04/09 15:41:48 bcotton Exp $
#-----------------------------------------------------------------------
#  Copyright (C) 2006
#  Associated Universities, Inc. Washington DC, USA.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge,
#  MA 02139, USA.
#
#  Correspondence concerning this software should be addressed as follows:
#         Internet email: bcotton@nrao.edu.
#         Postal address: William Cotton
#                         National Radio Astronomy Observatory
#                         520 Edgemont Road
#                         Charlottesville, VA 22903-2475 USA
#-----------------------------------------------------------------------

# Python interface to FITS directory utilities
import Obit, OSystem, OErr, os, pydoc, string

# Get list of FITS disks
FITSdisks = []
for dsk in ["FITS","FITS01","FITS02","FITS03","FITS04","FITS05","FITS06"]:
    dir = os.getenv(dsk)
    if dir:
        FITSdisks.append(dir)
        nFITS = len(FITSdisks)
del dsk, dir
        
def PListDir(disk):
    """ List files in FITS directory

    disk     = FITS disk number
    """
    ################################################################
    flist = os.listdir(FITSdisks[disk-1])
    dirlist = "FITS Directory listing for disk "+str(disk)+\
              ":"+FITSdisks[disk-1]+"\n"
    i=0
    for f in flist:
        dirlist = dirlist +string.rjust(str(i),6)+" "+f+"\n"
        i = i+1
    # User pager
    pydoc.ttypager(dirlist)
    # end PListDir


def PGetDir(disk):
    """ Returns list of files in FITS directory

    disk     = FITS disk number
    """
    ################################################################
    flist = os.listdir(FITSdisks[disk-1])
    return flist
    # end PGetDir


def PExist(file, disk, err):
    """ Tests if FITS file exists

    return True if exists, else False
    file     = FITS file name
    disk     = FITS disk number
    """
    ################################################################
    exist = Obit.FITSFileExist(file, disk, err.me)
    return exist!=0
    # end PExist

