# $Id: OPlot.py,v 1.2 2006/05/23 14:20:09 bcotton Exp $
#-----------------------------------------------------------------------
#  Copyright (C) 2006
#  Associated Universities, Inc. Washington DC, USA.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge,
#  MA 02139, USA.
#
#  Correspondence concerning this software should be addressed as follows:
#         Internet email: bcotton@nrao.edu.
#         Postal address: William Cotton
#                         National Radio Astronomy Observatory
#                         520 Edgemont Road
#                         Charlottesville, VA 22903-2475 USA
#-----------------------------------------------------------------------

# Python shadow class to ObitPlot class
import Obit, InfoList, Image

class OPlotPtr :
    def __init__(self,this):
        self.this = this
    def __setattr__(self,name,value):
        if name == "me" :
            # Out with the old
            Obit.OPlotUnref(Obit.OPlot_me_get(self.this))
            # In with the new
            Obit.OPlot_me_set(self.this,value)
            return
        self.__dict__[name] = value
    def __getattr__(self,name):
        if self.__class__ != OPlot:
            return
        if name == "me" : 
            return Obit.OPlot_me_get(self.this)
        # Functions to return members
        if name=="List":
            return PGetList(self)
        raise AttributeError,name
    def __repr__(self):
        if self.__class__ != OPlot:
            return
        return "<C OPlot instance> " + Obit.OPlotGetName(self.me)

class OPlot(OPlotPtr):
    """ Python Obit interface to display server
    
    This class is for creating and using the interface to a plot
    Image Members with python interfaces:
    InfoList  - used to pass instructions to processing
                Member List 
    """
    def __init__(self, name):
        self.this = Obit.new_OPlot(name)
    def __del__(self):
        if Obit!=None:
            Obit.delete_OPlot(self.this)

def newOPlot(name, err, file="None" ):
    """ Create and initialize an ObitPlot

    name     = name desired for object (labeling purposes)
    err      = Python Obit Error/message stack
    file     = name and type of output device:
               "None", interactive prompt
               /xterm  for an xterm (visual display Tektronics 4012 emulation)
               otherwise postscript file.
    """
    ################################################################
    out = OPlot(name)
    # Add "/PS" for postscript files
    if file and file!="/xterm":
        filename = file+"/PS"
    else:
        filename = file
    Obit.PlotInitPlot(out.me, filename, err.me)
    return out 
    # end newOPlot

def PXYPlot (plot, symbol, x, y, err):
    """ Simple XY Plot

    Plot X vs Y using symbol.
    Plot should be finalized and displayed with PShow
    This routine draws the frame and adds labels, to only overplot data
    on the same frame, use ObitPlotXYOver

    plot     = plot
    symbol   = Symbol index to use for plotting (pgplot)
       values in the range [1,31] are usable 
       if negative, use abs value and connect points
       1 = dot
       2 = plus
       3 = *
       4 = open circle
       5 = x
       6 = open square
       7 = open triangle
       12 = open star
       13 = filled triangle
       16 = filled square
       17 = filled circle
       18 = filled star
    x   =    Independent variable, if None use index
    y   =    Dependent variable
    err =    ObitErr error stack
    
    Optional parameters on plot InfoList
    XMAX (float) maximum X value (defaults to actual value)
    XMIN (float) minimum X value (defaults to actual value)
    YMAX (float) maximum Y value (defaults to actual value)
    YMIN (float) minimum Y value (defaults to actual value)
    TITLE (string)  Label for the plot (defaults to none), max 120
    XLABEL (string) Label for horizontal axis (defaults to none)
    XOPT   (string) Options for horizontal axis (default "BCNTS")
          See #ObitPlotCpgbox for details.
    YLABEL (string) Label for vertical axis (defaults to none)
    YOPT   (string) Options for  vertical axis (default "BCNTS")
         See #ObitPlotCpgbox for details.
    CSIZE  (int)    Scaling factor for characters(default = 1)
    LWIDTH (int)    Line width (default = 1)
    JUST   (int)    If !=0 then force X and Y axis scaling to be the same
    """
    ################################################################
    # Checks
    if not PIsA(plot):
        print "Actually ",plot.__class__
        raise TypeError,"plot MUST be a Python Obit Plot"
    n = len(y)  # How many points?
    Obit.PlotXYPlot (plot.me, symbol, n, x, y, err.me)
    # end PXYPlot


def PXYOver (plot, symbol, x, y, err):
    """ Overplot X vs Y 

    Overplot X vs Y using symbol.
    Plot should be finalized and displayed with PShow

    plot     = plot
    symbol   = Symbol index to use for plotting (pgplot)
       values in the range [1,31] are usable 
       if negative, use abs value and connect points
       1 = dot
       2 = plus
       3 = *
       4 = open circle
       5 = x
       6 = open square
       7 = open triangle
       12 = open star
       13 = filled triangle
       16 = filled square
       17 = filled circle
       18 = filled star
    x   =    Independent variable, if None use index
    y   =    Dependent variable
    err =    ObitErr error stack
    """
    ################################################################
    # Checks
    if not PIsA(plot):
        print "Actually ",plot.__class__
        raise TypeError,"plot MUST be a Python Obit Plot"
    n = len(y)  # How many points?
    Obit.PlotXYOver (plot.me, symbol, n, x, y, err.me)
    # end PXYOver

def PXYErr (plot, symbol, x, y, e, err):
    """ Simple XY Plot

    Plot X vs Y using symbol.
    Plot should be finalized and displayed with PShow
    This routine draws the frame and adds labels, to only overplot data
    on the same frame, use ObitPlotXYOver

    plot     = plot
    symbol   = Symbol index to use for plotting (pgplot)
       values in the range [1,31] are usable 
       if negative, use abs value and connect points
       1 = dot
       2 = plus
       3 = *
       4 = open circle
       5 = x
       6 = open square
       7 = open triangle
       12 = open star
       13 = filled triangle
       16 = filled square
       17 = filled circle
       18 = filled star
    x   =    Independent variable, if None use index
    y   =    Dependent variable
    e   =    if nonNone, error in y
    err =    ObitErr error stack
    
    Optional parameters on plot InfoList
    XMAX (float) maximum X value (defaults to actual value)
    XMIN (float) minimum X value (defaults to actual value)
    YMAX (float) maximum Y value (defaults to actual value)
    YMIN (float) minimum Y value (defaults to actual value)
    TITLE (string)  Label for the plot (defaults to none), max 120
    XLABEL (string) Label for horizontal axis (defaults to none)
    XOPT   (string) Options for horizontal axis (default "BCNTS")
          See #ObitPlotCpgbox for details.
    YLABEL (string) Label for vertical axis (defaults to none)
    YOPT   (string) Options for  vertical axis (default "BCNTS")
         See #ObitPlotCpgbox for details.
    CSIZE  (int)    Scaling factor for characters(default = 1)
    LWIDTH (int)    Line width (default = 1)
    JUST   (int)    If !=0 then force X and Y axis scaling to be the same
    """
    ################################################################
    # Checks
    if not PIsA(plot):
        print "Actually ",plot.__class__
        raise TypeError,"plot MUST be a Python Obit Plot"
    n = len(y)  # How many points?
    Obit.PlotXYErr (plot.me, symbol, n, x, y, e, err.me)
    # end PXYErr


def PContour (plot, label, image, lev, cntfac, err):
    """ Contour plot of image

    Contours at lev times powers of cntfac
    NOTE: THIS uses pgplot which doesn't correctly handle rotated images
    NOTE: Since pgplot only has flat geometries, this doesn't work well
    for larger fields of view. :'<
    Plot should be finalized and displayed with PShow
    plot     = plot
    label    = Label for plot
    image    = ObitImage to plot
    lev      = basic contour level (def 0.1 peak)
    cntfac   = factor for spacing between contours (def sqrt(2)
    err      =    ObitErr error stack
    
    Optional parameters on plot InfoList
    CSIZE  (int)    Scaling factor for characters(default = 1)
    LWIDTH (int)    Line width (default = 1)
    """
    ################################################################
    # Checks
    if not PIsA(plot):
        print "Actually ",plot.__class__
        raise TypeError,"plot MUST be a Python Obit Plot"
    if not Image.PIsA(image):
        print "Actually ",image.__class__
        raise TypeError,"image MUST be a Python Obit Image"
    Obit.PlotContour (plot.me, label, image.me, lev, cntfac, err.me)
    # end PContour


def PMarkCross (plot, image, ra, dec, err, size=5.0):
    """ Mark positions on Contour plot of image

    Place cross at positions.
    Plot should be finalized and displayed with PShow
    plot     = plot
    image    = ObitImage to plot
    ra       = list of RAs (deg)
    dec      = list of Declinations (deg)
    err      =    ObitErr error stack
    size     = size of cross in pixels
    
    Optional parameters on plot InfoList
    CSIZE  (int)    Scaling factor for characters(default = 1)
    LWIDTH (int)    Line width (default = 1)
    """
    ################################################################
    # Checks
    if not PIsA(plot):
        print "Actually ",plot.__class__
        raise TypeError,"plot MUST be a Python Obit Plot"
    if not Image.PIsA(image):
        print "Actually ",image.__class__
        raise TypeError,"image MUST be a Python Obit Image"
    n = len(ra)
    Obit.PlotMarkCross (plot.me, image.me, n, ra, dec, size, err.me)
    # end PMarkCross

def PShow (plot, err):
    """ Display plot

    plot   = Python Plot object
    err =    ObitErr error stack
    """
    ################################################################
     # Checks
    if not PIsA(plot):
        raise TypeError,"plot MUST be a Python Obit plot"
    #
    Obit.PlotFinishPlot(plot.me, err.me)
    # end  PShow


def PGetList (plot):
    """ Return the member InfoList

    returns InfoList
    plot   = Python Obit Plot object
    """
    ################################################################
     # Checks
    if not PIsA(plot):
        raise TypeError,"plot MUST be a Python Obit plot"
    #
    out    = InfoList.InfoList()
    out.me = Obit.InfoListUnref(out.me)
    out.me = Obit.PlotGetList(plot.me)
    return out
    # end PGetList

def PIsA (plot):
    """ Tells if the input is a Python ObitPlot

    returns true or false (1,0)
    plot = Python Obit Plot to test
    """
    ################################################################
      # Checks
    if plot.__class__ != OPlot:
        return 0
    return Obit.OPlotIsA(plot.me)
    # end PIsA


